% Kostis Hatzitaskos
% Ruud estimation prep file - ranking choice
% June 24 2003

% Initialization ----------------------------------------------------------
clc;
clear;
clear global;
pack;
mtime=cputime;                  % Start Timing
rand('state',sum(100*clock));   % Randomize rand
delete('log_prep.txt');
diary('log_prep.txt');

fprintf('\n');
fprintf('----------- Ruud Estimation Data Preparation -----------');
fprintf('\n\n');

% Data Prep ---------------------------------------------------------------
Y=1;                            % Must enter correct number of years
X=load('char.txt');
J=length(X)/Y;                  % Number of journals
hold=load('hold.txt');
S=(length(hold)/J)/Y;           % Number of schools
hldn=reshape(hold(:,1),J,S,Y);
hldp=reshape(hold(:,2),J,S,Y);
matcho=sum(hldn);

fprintf('\n');
fprintf('%.0f Schools, %.0f Journals, %.0f Years',S,J,Y);
fprintf('\n\n');

save stat J S Y;
fprintf('stat file saved\n');
save data X hldn hldp matcho;
fprintf('data file saved\n');
fprintf('\n');

% Ranking Choice ----------------------------------------------------------

fprintf('\n');
fprintf('--------- Ruud Estimation Naive Ranking Choice ---------');
fprintf('\n\n');

M=input('Enter maximum number of rankings to be used: ');
fprintf('\n');
save ranks M;

% The maximum number of rankings that will be generated. A good number is
% 720=6!, hence libraries that use choose up to six journals will be
% estimated with no simulation error.

% Idealy we'd use every single permutation of the chosen and non-chosen
% journals. For libraries that choose more than M journals, we will pick M
% distinct rankings for the chosen set.

% A denotes the chosen set, R is the ranking matrix

% To increase estimation efficiency, the algorithm below only generates
% distinct rankings. Since this is still done randomly, however, it may
% take some time.

for y=1:Y;
    numcho=matcho(:,:,y);
    for s=1:S;
        if numcho(s)==J;
            fprintf('School %4.0f --  J chosen in year %.0f, zero rankings generated\n',s,y);
            continue;
        end;
        if numcho(s)==0;
            fprintf('School %4.0f --  0 chosen in year %.0f, zero rankings generated\n',s,y);
            continue;
        end;
        Rref=['Rs',int2str(s),'y',int2str(y)];
        cardA=factorial(numcho(s));     % Cardinality of A
        numrank=min(M,cardA);           % Number of rankings
        Rini=['R=zeros(',int2str(numcho(s)),',',int2str(numrank),');'];
        eval(Rini);
        drop=hldn(:,s,y)+rand(J,1);
        [drop,z]=sort(-drop);
        R(:,1)=z(1:numcho(s));
        for r=2:numrank;
            ok=0;
            while ok==0;
                repeated=0;
                drop=hldn(:,s,y)+rand(J,1);
                [drop,z]=sort(-drop);
                for n=1:r-1;
                    if z(1:numcho(s))==R(:,n);
                        repeated=1;
                    end;
                end;
                if repeated~=1;
                    ok=1;
                end;            
            end;
            R(:,r)=z(1:numcho(s));
        end;
        Rsav=['save ',Rref,' R'];
        eval(Rsav);
        fprintf('School %4.0f -- %2.0f chosen in year %.0f, %4.0f rankings generated (out of %.5g)\n',s,numcho(s),y,numrank,cardA);
    end;
end;

% Wrapup ------------------------------------------------------------------
fprintf('\n');
mtime=cputime-mtime;
mtimem=fix(mtime/60);
mtimes=rem(mtime,60);
mtimeh=fix(mtimem/60);
mtimem=rem(mtimem,60);
fprintf('------------------------------------------------------\n');
fprintf('.m file execution time %.0fhrs, %.0fmin, %5.4gsec\n',mtimeh,mtimem,mtimes);
fprintf('------------------------------------------------------\n');

beep;
diary off;